const fs = require('fs');
const path = require('path');

const configPath = path.join(__dirname, '../config/config.json');
const hostConfigPath = path.join(__dirname, '../config/hostconfig.json');

let currentConfig = null;
let previousConfig = null;

const configManager = {
	load: function() {
		try {
			if (!fs.existsSync(configPath)) {
				throw new Error('Missing config.json.');
			}
			const configData = JSON.parse(fs.readFileSync(configPath, 'utf-8'));

			if (!fs.existsSync(hostConfigPath)) {
				fs.writeFileSync(hostConfigPath, JSON.stringify({ mappings: [] }, null, 2));
			}
			const hostConfigData = JSON.parse(fs.readFileSync(hostConfigPath, 'utf-8'));

			previousConfig = { ...hostConfigData }; // Clone for rollback
			currentConfig = hostConfigData;
			      
			return { config: configData, hostConfig: hostConfigData };
		} catch (err) {
			console.error('Error loading configuration:', err.message);
			throw err;
		}
	},
	  
	saveHostConfig: function(newConfig) {
		try {
			this.validate(newConfig);
			previousConfig = { ...currentConfig };
			currentConfig = newConfig;
			fs.writeFileSync(hostConfigPath, JSON.stringify(newConfig, null, 2));
			console.log('Host configuration updated successfully.');
		} catch (err) {
			console.error('Error saving host configuration:', err.message);
		}
	},

	validate: function(config) {
		if (!Array.isArray(config)) {
			throw new Error('Invalid configuration: Config must be an array.');
		}
		config.forEach((mapping, index) => {
			if (!mapping.host || typeof mapping.host !== 'string') {
				throw new Error(`Invalid mapping at index ${index}: "host" must be a string.`);
			}
			if (!mapping.to || typeof mapping.to !== 'string') {
				throw new Error(`Invalid mapping at index ${index}: "to" must be a string.`);
			}
		});
	},

	getPreviousConfig: function() {
		return previousConfig;
	},

	getCurrentConfig: function() {
		return currentConfig;
	},
};

module.exports = configManager;
