const http = require('http');
const https = require('https');
const fs = require('fs');
const path = require('path');
const flatted = require('flatted');
const configManager = require('./configManager');
const { config, hostConfig } = configManager.load();


const routeManager = {
	setupRoutes: function() {
		config.ports.forEach((port) => {
			const server = (port == 443 ? https : http).createServer(this.requestHandler);

			server.listen(port, () => {
				console.log(`${port == 443 ? 'HTTPS' : 'HTTP'} Server listening on port ${port}`);
			});
		});
	},

	requestHandler: function(req, res) {
	    const hostEntry = hostConfig.find(config => config.host === req.headers.host);
	  
	    if (!hostEntry) {
	        res.writeHead(403, { 'Content-Type': 'text/plain' });
	        res.end('Forbidden: Invalid Host');
	        return;
	    }
	  
	    const url = new URL(req.url, `http://${hostEntry.to}`);

		let options = {
	        method: req.method,
	        headers: req.headers,
	    };
	    
	    options.headers['Host'] = req.headers.host;
	    options.headers['X-Real-IP'] = req.headers['x-real-ip'] || req.connection.remoteAddress;
	    options.headers['X-Forwarded-For'] = req.headers['x-forwarded-for'] || req.connection.remoteAddress;
	    options.headers['X-Forwarded-Proto'] = req.headers['x-forwarded-proto'] || req.protocol || 'http';

		const proxyRequest = http.request(url, options, (proxyResponse) => {
		    const editedHeaders = { ...proxyResponse.headers };

		    delete editedHeaders['server'];
		    if (config.setServer) {
		        editedHeaders['Server'] = config.customServer || 'WWN/1.0';
		    } else {
		        delete editedHeaders['Server'];
		    }

		    delete editedHeaders['x-powered-by'];
		    if (config.setXPoweredBy) {
		        editedHeaders['X-Powered-By'] = config.customXPoweredBy || 'WWN/1.0';
		    } else {
		        delete editedHeaders['X-Powered-By'];
		    }

		    res.writeHead(proxyResponse.statusCode, editedHeaders);
		    proxyResponse.pipe(res, { end: true });
		});

	    proxyRequest.on('error', (err) => {
	        console.error('Error with the proxy request:', err);
	        res.writeHead(500);
	        res.end('Internal Server Error');
	    });

		req.pipe(proxyRequest, { end: true });
	}
};

module.exports = routeManager;
