const WebSocket = require('ws');
const configManager = require('./configManager');
const routeManager = require('./routeManager');
const serverManager = require('./serverManager');

const wsManager = {
	start: function(config) {
		const wss = new WebSocket.Server({ port: 64999 });

		if (config.CWS_Allow_Everyone) {
			console.warn("YOU ARE RUNNING WEBSOCKET IN INSECURE, ALL-IP-ALLOWED MODE.");
		}

		wss.on('connection', (ws, req) => {
			if (!config.CWS_Allowed_IPs.includes(req.socket.remoteAddress) && !config.CWS_Allow_Everyone) {
				console.log(`${req.socket.remoteAddress} tried to connect to Config WebSocket, but such IP is NOT in the config.`);
				console.log('If this was you, update config and reload WWN');
				ws.close(1008, 'Forbidden');
				return;
			}

			ws.on('message', (message) => {
				let data;
				try {
					data = JSON.parse(message);
				} catch (err) {
					ws.send('Invalid message. Must be JSON');
					return;
				}

				try {
				if (data.command === 'prevconf') {
					ws.send(JSON.stringify(configManager.getPreviousConfig()));
				} else if (data.command === 'currconf') {
					ws.send(JSON.stringify(configManager.getCurrentConfig()));
				} else if (data.command === 'quit') {
					ws.close(1000, 'Clean exit');
				} else if (data.config) {
					configManager.saveHostConfig(data.config);
					ws.send('Configuration updated, requesting reload...');
					serverManager.reloadRequest();
				} else {
					ws.send('Unknown command.')
				}
				} catch (err) {
					ws.send(`Error: ${err.message}`);
				}
			});
		});

		console.log('WebSocket server running on port 64999.');
	},
};

module.exports = wsManager;
